{----------------------------------------------------}
{ Andreis LCARS-Components 1.2 for Delphi 2, (3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSEdge;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TEdgePosition = (leTopLeft, leTopRight, leBottomLeft, leBottomRight);
  TLCARSEdge = class(TGraphicControl)
  private
   FEdgePosition : TEdgePosition;
   FEdgeColor : TColor;
   FThicknessX : integer;
   FThicknessY : integer;
   FOuterRadiusX : integer;
   FOuterRadiusY : integer;
   FInnerRadiusX : integer;
   FInnerRadiusY : integer;
   FInnerFull : boolean;
   FBack : TBitmap;
   FBackColor : TColor;
   procedure SetEdgePosition(value : TEdgePosition);
   procedure SetEdgeColor(value : TColor);
   procedure SetThicknessX(value : integer);
   procedure SetThicknessY(value : integer);
   procedure SetInnerFull(value : boolean);
   procedure SetOuterRadiusX(value : integer);
   procedure SetOuterRadiusY(value : integer);
   procedure SetInnerRadiusX(value : integer);
   procedure SetInnerRadiusY(value : integer);
  protected
   procedure Paint; override;
  public
   constructor Create(AOwner : TComponent) ; override;
   destructor Destroy; override;
  published
   property Anchors;
   property Constraints;
   property Parent;
   property Width default 100;
   property Height default 100;
   property Hint;
   property ShowHint;
   property ParentShowHint;
   property Align;
   property EdgeColor : TColor read FEdgeColor
    write SetEdgeColor default clyellow;
   property Visible default true;
   property EdgePosition : TEdgePosition read FEdgePosition
    write SetEdgePosition default leTopLeft;
   property ThicknessX : integer read FThicknessX
    write SetThicknessX default 10;
   property ThicknessY : integer read FThicknessY
    write SetThicknessY default 10;
   property InnerFull : boolean read FInnerFull
    write SetInnerFull default false;
   property OuterRadiusX : integer read FOuterRadiusX
    write SetOuterRadiusX default 50;
   property OuterRadiusY : integer read FOuterRadiusY
    write SetOuterRadiusY default 50;
   property InnerRadiusX : integer read FInnerRadiusX
    write SetInnerRadiusX default 50;
   property InnerRadiusY : integer read FInnerRadiusY
    write SetInnerRadiusY default 50;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

procedure Register;

implementation

constructor TLCARSEdge.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 100;
 Height := 100;
 ThicknessX := 10;
 ThicknessY := 10;
 InnerFull := false;
 OuterRadiusX := 50;
 OuterRadiusY := 50;
 InnerRadiusX := 50;
 InnerRadiusY := 50;
 EdgeColor := clyellow;
 Visible := true;
 Canvas.Brush.Style := bsClear;
 ControlStyle:=ControlStyle+[csOpaque];
 FBack := TBitmap.Create;
end;

destructor TLCARSEdge.Destroy;
begin
 FBack.Free;
 inherited Destroy;
end;

procedure TLCARSEdge.Paint;
var VisibleRect, BackRect : TRect;
begin
 FBack.Width := Width * 2;
 FBack.Height := Height * 2;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0,0, FBack.width, FBack.height);
 FBack.Canvas.Brush.Color := EdgeColor;
 FBack.Canvas.Pen.Color := EdgeColor;
 FBack.Canvas.RoundRect(0,0, FBack.width, FBack.height,
  FOuterRadiusX,FOuterRadiusY);
 if InnerFull = false then
  begin
   FBack.Canvas.Brush.Color := FBackColor;
   FBack.Canvas.Pen.Color := FBackColor;
   FBack.Canvas.RoundRect(FThicknessX,FThicknessY,
    FBack.width-FThicknessX, FBack.height-FThicknessY,
    FInnerRadiusX,FInnerRadiusY);
  end;
 VisibleRect := Rect(0, 0, Width, Height);
 case FEdgePosition of
  leTopLeft :
   BackRect := Rect(0, 0, FBack.Width div 2, Fback.Height div 2);
  leTopRight :
   BackRect := Rect(FBack.width div 2, 0, FBack.Width, Fback.Height div 2);
  leBottomLeft :
   BackRect := Rect(0, FBack.height div 2, FBack.Width div 2, Fback.Height);
  leBottomRight :
   BackRect := Rect(FBack.Width div 2, Fback.Height div 2, FBack.Width, Fback.Height);
 end;
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSEdge.SetEdgePosition(value : TEdgePosition);
begin
 if FEdgePosition <> value then
  begin
   FEdgePosition := value;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetEdgeColor(value : TColor);
begin
 if FEdgeColor <> value then
  begin
   FEdgeColor := value;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetThicknessX(value : integer);
begin
 if FThicknessX <> value then
  begin
   if value < Width then FThicknessX := value
   else FThicknessX := Width - 1;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetThicknessY(value : integer);
begin
 if FThicknessY <> value then
  begin
   if value < Height then FThicknessY := value
   else FThicknessY := Height - 1;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetInnerFull(value : boolean);
begin
 if FInnerFull <> value then
  begin
   FInnerFull := value;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetOuterRadiusX(value : integer);
begin
 if FOuterRadiusX <> value then
  begin
   FOuterRadiusX := value;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetOuterRadiusY(value : integer);
begin
 if FOuterRadiusY <> value then
  begin
   FOuterRadiusY := value;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetInnerRadiusX(value : integer);
begin
 if FInnerRadiusX <> value then
  begin
   FInnerRadiusX := value;
   Repaint;
  end;
end;

procedure TLCARSEdge.SetInnerRadiusY(value : integer);
begin
 if FInnerRadiusY <> value then
  begin
   FInnerRadiusY := value;
   Repaint;
  end;
end;

procedure Register;
begin
  RegisterComponents('LCARS-Comps', [TLCARSEdge]);
end;

end.
